package algorithms.geneticAlgorithm.Components.Selections;

import java.util.ArrayList;
import java.util.Random;

import algorithms.geneticAlgorithm.Components.GAIndividual;
import algorithms.geneticAlgorithm.Components.GASelectionStrategy;

public class AdvancedSelectionStrategy<I extends GAIndividual> implements GASelectionStrategy<I>{
	
	public class Range{
		
		public double min;
		public double max;
		public Range(double min, double max){
			this.min = min;
			this.max = max;
		}
	}
	
	ArrayList<Range> rouletteWheel;
	ArrayList<I> currentPop;
	Random random;
	double maxOffset;
	double minRange;
	double accFittness;
	double highestFittness;
	double lowestFittness;
	
	public AdvancedSelectionStrategy(){
		rouletteWheel = new ArrayList<Range>();
		random = new Random();
		currentPop = new ArrayList<I>();
	}

	
	@Override
	public void setCurrentPopulation(ArrayList<I> currentPopulation) {
		currentPop = currentPopulation;
		rouletteWheel.clear();
		double minOffset = 0;
		maxOffset = 0;
		initValues();
		
		//y = a*x + b mit y zwischen 0 und 1 
		double a = 1/(highestFittness - lowestFittness);
		double b = -(a*lowestFittness);
		if(lowestFittness != highestFittness){
			//b += 1/highestFittness - lowestFittness;
		}
		
		for(I individual : currentPop){
			double space = a*individual.getFittness() + b;
			minOffset = maxOffset;
			maxOffset = minOffset + space;
			
			Range range = new Range(minOffset, maxOffset);
			rouletteWheel.add(range);
		}
	}
	
	public void initValues(){
		lowestFittness = currentPop.get(0).getFittness();
		highestFittness = currentPop.get(0).getFittness();
		
		for(I i : currentPop){
			if(i.getFittness() < lowestFittness){
				lowestFittness = i.getFittness();
			}
			if(i.getFittness() > highestFittness){
				highestFittness = i.getFittness();
			}
		}
	}

	
	@Override
	public ArrayList<I> selectIndividuals(){
		ArrayList<I> parents = new ArrayList<I>();
		parents.add(selectIndividual());
		parents.add(selectIndividual());
		return parents;
	}
	
	public I selectIndividual() {
		double randomValue = random.nextDouble() * maxOffset;
		
		for(int i = 0; i < rouletteWheel.size(); i++){
			Range current = rouletteWheel.get(i);
			if(current.min <= randomValue && randomValue < current.max){
				return currentPop.get(i);
			}
		}
		
		return currentPop.get(random.nextInt(currentPop.size() - 1));
	}

	
	

}
